import { Http, IDictionary, IRequestConfig } from '@zeedhi/core';
import { RequestLogger } from './request-logger';
import routeHandlers from './routeHandlers';

interface RouteStructure {
    resource: string;
    uniqueKey: string;
    operation: 'create' | 'read' | 'update' | 'delete';
}

/**
 * Classe para realizar a conversão de requisições para o padrão REST
 */
export class HttpCRUDTranslator {
    // Mapeamento entre rotas do back - rotas do mirage
    private static routeMap: IDictionary<RouteStructure> = {
        '/getMinhasTarefas': {
            resource: 'minhasTarefas', uniqueKey: 'NRCHECKLISTAGEND', operation: 'read',
        },
        '/getItensTarefas': {
            resource: 'itensTarefas', uniqueKey: 'NRTAREFAAGEND', operation: 'read',
        },
        '/getOrdenacao': {
            resource: 'ordenacao', uniqueKey: 'NRORDENCAOAPP', operation: 'read',
        },
        '/insertOrdenacao': {
            resource: 'ordenacao', uniqueKey: 'NRORDENCAOAPP', operation: 'create',
        },
        '/alterarStatusTarefa': {
            resource: 'minhasTarefas', uniqueKey: 'NRCHECKLISTAGEND', operation: 'update',
        },
        '/finalizarTarefa': {
            resource: 'minhasTarefas', uniqueKey: 'NRCHECKLISTAGEND', operation: 'update',
        },
        '/alterarStatusItem': {
            resource: 'itensTarefas', uniqueKey: 'NRTAREFAAGEND', operation: 'update',
        },
        '/getTarefasListadas': {
            resource: 'tarefasListadas', uniqueKey: 'NRCHECKLISTAGEND', operation: 'read',
        },
        '/getEpisTarefa': {
            resource: 'episTarefa', uniqueKey: 'NRRELCHKAGENDEPI', operation: 'read',
        },
        '/salvarObservacao': {
            resource: 'minhasTarefas', uniqueKey: 'NRCHECKLISTAGEND', operation: 'update',
        },
        '/getGeolocalizacao': {
            resource: 'locais', uniqueKey: 'NRLOCALCLIENTE', operation: 'read',
        },
        '/getGeolocalizacaoParametrizacao': {
            resource: 'parametrizacao', uniqueKey: 'NRPARAMGERAL', operation: 'read',
        },
        '/anexarImagensTarefas': {
            resource: 'imagensTarefas', uniqueKey: 'NRFOTOTAREFAAGEND', operation: 'create',
        },
        '/excluirFotoTarefas': {
            resource: 'imagensTarefas', uniqueKey: 'NRFOTOTAREFAAGEND', operation: 'delete',
        },
        '/getTarefasCalendario': {
            resource: 'minhasTarefas', uniqueKey: 'NRCHECKLISTAGEND', operation: 'read',
        },
        '/getVistoriasListadas': {
            resource: 'vistorias', uniqueKey: 'NRGERENCIAMENTOCHK', operation: 'read',
        },
        '/getParamOffline': {
            resource: 'parametrizacao', uniqueKey: 'NRPARAMGERAL', operation: 'read',
        },
        '/getModeloPopTarefa': {
            resource: 'parametrizacao', uniqueKey: 'NRPARAMGERAL', operation: 'read',
        },
        '/getCriterios': {
            resource: 'criterios', uniqueKey: 'NRCRITERIOTARGERCHK', operation: 'read',
        },
        '/getAtividades': {
            resource: 'atividades', uniqueKey: 'NRATIVIDADECHK', operation: 'read',
        },
        '/getAprovacoes': {
            resource: 'aprovacoes', uniqueKey: 'NRCHECKLISTAGEND', operation: 'read',
        },
        '/getPrincipaisNaoConformidades': {
            resource: 'principaisNaoConformidades', uniqueKey: 'NRAUDITORIA', operation: 'read',
        },
        '/getNaoConformidadesGerais': {
            resource: 'naoConformidadesGerais', uniqueKey: 'NRAUDITORIA', operation: 'read',
        },
        '/getAuditorias': {
            resource: 'auditorias', uniqueKey: 'NRAUDITORIA', operation: 'read',
        },
        '/getCategoriasAuditoria': {
            resource: 'categoriasAuditoria', uniqueKey: 'NRCATEGORIAAUDIT', operation: 'read',
        },
        '/getCriteriosAuditoria': {
            resource: 'criteriosAuditoria', uniqueKey: 'NRTOPICOAUDIT', operation: 'read',
        },
        '/getTopicosAuditoria': {
            resource: 'topicosAuditoria', uniqueKey: 'NRRELACRESULTCRIT', operation: 'read',
        },
        '/inicarOuFinalizarAuditoria': {
            resource: 'auditorias', uniqueKey: 'NRAUDITORIA', operation: 'update',
        },
        '/CategoriaManut': {
            resource: 'categoriaManut', uniqueKey: 'NRCATEGORIAMANUT', operation: 'read',
        },
        '/getEquipamento': {
            resource: 'equipamentos', uniqueKey: 'NREQUIPAMENTO', operation: 'read',
        },
        '/getSolicitacoes': {
            resource: 'solicitacoes', uniqueKey: 'NRSOLICITACAO', operation: 'read',
        },
        '/getAvaliacao': {
            resource: 'avaliacoes', uniqueKey: 'IDSATISFACAOSOLICITA', operation: 'read',
        },
        '/criarSolicitacao': {
            resource: 'solicitacoes', uniqueKey: 'NRSOLICITACAO', operation: 'create',
        },
        '/tarefasFinalizadas': {
            resource: 'tarefasFinalizadas', uniqueKey: 'NRCHECKLISTAGEND', operation: 'read',
        },
        '/dataBaseChartPie': {
            resource: 'dataBaseChartPie', uniqueKey: 'NMATIVIDADE', operation: 'read',
        },
        '/dataBaseChartBar': {
            resource: 'dataBaseChartBar', uniqueKey: 'NRCRITERIOTAREFACHK', operation: 'read',
        },
        '/tarefasAtrasadas': {
            resource: 'tarefasAtrasadas', uniqueKey: 'NRCHECKLISTAGEND', operation: 'read',
        },
        '/tarefasConcluidas': {
            resource: 'tarefasConcluidas', uniqueKey: 'NRCHECKLISTAGEND', operation: 'read',
        },
        '/ultimasTarefasRealizadas': {
            resource: 'ultimasTarefasRealizadas', uniqueKey: 'CODIGO', operation: 'read',
        },
        '/OrdensServicosDash': {
            resource: 'ordensServico', uniqueKey: 'NRORDEMSERV', operation: 'read',
        },
        '/SolicitacoesServicosDash': {
            resource: 'solicitacoesServico', uniqueKey: 'NRSOLICITACAOSERV', operation: 'read',
        },
        '/avaliarSolicitacao': {
            resource: 'avaliacoes', uniqueKey: 'NRSOLICITACAO', operation: 'create',
        },
        '/getFotosSolicitacao': {
            resource: 'fotosSolicitacao', uniqueKey: 'NRIMGSOLICITACAO', operation: 'read',
        },
        '/Funcionarios/PerfilAcesso': {
            resource: 'perfilAcesso', uniqueKey: 'NRFUNCIONARIO', operation: 'read',
        },
        '/getImpactoManutencao': {
            resource: 'impactos', uniqueKey: 'NRIMPACTOMANUT', operation: 'read',
        },
        '/qtdTarefasAtivas': {
            resource: 'impactos', uniqueKey: 'TAREFAATIVA', operation: 'read',
        },
        '/SolicitacoesConvertidas': {
            resource: 'solicitacoesConvertidas', uniqueKey: 'NRSOLICITACAOSERV', operation: 'read',
        },
        '/OrdensPendente': {
            resource: 'ordensPendente', uniqueKey: 'NRORDEMSERV', operation: 'read',
        },
        '/SolicitacaoServ': {
            resource: 'solicitacoes', uniqueKey: 'NRSOLICITACAOSERV', operation: 'create',
        },
        '/getImagensAuditoria': {
            resource: 'imagensAuditoria', uniqueKey: 'NRFOTORESULTAUDIT', operation: 'read',
        },
        '/anexarImagemAuditoria': {
            resource: 'imagensAuditoria', uniqueKey: 'NRFOTORESULTAUDIT', operation: 'create',
        },
        '/getFotosVistorias': {
            resource: 'fotosVistoria', uniqueKey: 'NRFOTOTAREFAGERCHK', operation: 'read',
        },
        '/getFotosAprovacao': {
            resource: 'fotosAprovacao', uniqueKey: '', operation: 'read',
        },
        // aqui podemos adicionar rotas de outros cruds
    };

    private static buildQueryStringOff(data: any[]): string {
        return data ? data.reduce((result: string, filter: any) => {
            if (Array.isArray(filter.value)) {
              const values = filter.value.map((val: any) => `${filter.name}=${val}`).join('&');
              return `${result}${values}&`;
            }
            return `${result}${filter.name}=${filter.value}&`;
          }, '?').slice(0, -1)
        : '';
    }

    private static buildQueryStringOn(data: any[]): string {
        return data ? data.reduce((result: string, filter: any) => {
            if (Array.isArray(filter.value)) {
              const values = filter.value.map((val: any) => `${filter.name}[]=${val}`).join('&');
              return `${result}${values}&`;
            }
            return `${result}${filter.name}=${filter.value}&`;
          }, '?').slice(0, -1)
        : '';
    }

    public static async post(url: string, data?: any, config?: IRequestConfig): Promise<any> {
        if (navigator.onLine) return Http.post(url, data, config);

        if (routeHandlers[url]) {
            const response = routeHandlers[url](data);
            RequestLogger.logRequest(url, data, 'post');
            return response;
        }

        const structure = this.routeMap[url];
        if (!structure) {
            return Promise.reject(new Error(`Rota nao Definida no RouteMap ${url}`));
        }
        const {
            resource, uniqueKey, operation,
        } = structure;

        try {
            let response;

            if (operation === 'read') {
                const query = this.buildQueryStringOff(data);
                response = await Http.get(`${resource}${query}`, config);
            } else if (operation === 'create') {
                if (!Array.isArray(data)) {
                    response = await Http.post(`${resource}`, data);
                } else {
                    response = await Promise.all(data.map((row) => Http.post(resource, row)));
                }
            } else if (operation === 'update') {
                if (!Array.isArray(data)) {
                    response = await Http.put(`${resource}/${data[uniqueKey]}`, data);
                } else {
                    response = await Promise.all(data.map((row) => Http.put(`${resource}/${row[uniqueKey]}`, row)));
                }
            } else if (operation === 'delete') {
                response = await Http.delete(`${resource}/${data[uniqueKey]}`, config);
            } else {
                throw new Error(`Esta rota nao esta preparada para esse metodo ${url}`);
            }

            RequestLogger.logRequest(url, data, 'post'); // Somente após sucesso
            return response;
        } catch (error) {
            return Promise.reject(error);
        }
    }

    public static get(url: string, data?: any, config?: IRequestConfig): Promise<any> {
        if (navigator.onLine) {
            const query = this.buildQueryStringOn(data);
            return Http.get(`${url}${query}`, config);
        }

        if (routeHandlers[url]) {
            return routeHandlers[url](data, config);
        }

        const structure = this.routeMap[url];
        if (!structure) {
            return Promise.reject(new Error(`Rota nao Definida no RouteMap ${url}`));
        }
        const {
            resource, operation,
        } = structure;

        // Trata operação de read, convertendo o filtro para query string
        if (operation === 'read') {
            const query = this.buildQueryStringOff(data);
            return Http.get(`${resource}${query}`, config);
        }
        // Se nenhuma operação for encontrada, apenas retorna null
        return Promise.reject(new Error(`Esta rota nao esta preparada para esse metodo ${url}`));
    }
}
